@@ -150,7 +150,7 @@ function generate_containerized_kuryr_resources {
 
     # Generate kuryr resources in k8s formats.
     local output_dir="${DATA_DIR}/kuryr-kubernetes"
-    generate_kuryr_configmap $output_dir $KURYR_CONFIG $KURYR_CONFIG
+    generate_kuryr_configmap $output_dir $KURYR_CONFIG
     generate_kuryr_certificates_secret $output_dir $SSL_BUNDLE_FILE
     generate_kuryr_service_account $output_dir
     generate_controller_deployment $output_dir $KURYR_HEALTH_SERVER_PORT $KURYR_CONTROLLER_HA
@@ -273,15 +273,24 @@ function configure_neutron_defaults {
     local ext_svc_subnet_id
     local prot
 
+    project_id=$(get_or_create_project \
+        "$KURYR_NEUTRON_DEFAULT_PROJECT" default)
+    ext_svc_net_id="$(openstack network show -c id -f value \
+        "${KURYR_NEUTRON_DEFAULT_EXT_SVC_NET}")"
+
     # If a subnetpool is not passed, we get the one created in devstack's
     # Neutron module
     subnetpool_id=${KURYR_NEUTRON_DEFAULT_SUBNETPOOL_ID:-${SUBNETPOOL_V4_ID}}
     router=${KURYR_NEUTRON_DEFAULT_ROUTER:-$Q_ROUTER_NAME}
+    if [ "$router" != "$Q_ROUTER_NAME" ]; then
+        openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
+            router create --project "$project_id" "$router"
+        openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
+            router set --external-gateway "$ext_svc_net_id" "$router"
+    fi
     router_id="$(openstack router show -c id -f value \
         "$router")"
 
-    project_id=$(get_or_create_project \
-        "$KURYR_NEUTRON_DEFAULT_PROJECT" default)
     create_k8s_subnet "$project_id" \
                       "$KURYR_NEUTRON_DEFAULT_POD_NET" \
                       "$KURYR_NEUTRON_DEFAULT_POD_SUBNET" \
@@ -304,12 +313,6 @@ function configure_neutron_defaults {
             --project "$project_id" -c ID -f value) | tr ' ' ',')
     fi
 
-    ext_svc_net_id="$(openstack network show -c id -f value \
-        "${KURYR_NEUTRON_DEFAULT_EXT_SVC_NET}")"
-
-    ext_svc_subnet_id="$(openstack subnet show -c id -f value \
-        "${KURYR_NEUTRON_DEFAULT_EXT_SVC_SUBNET}")"
-
     # In order for the ports to allow service traffic under Octavia L3 mode,
     # it is necessary for the service subnet to be allowed into the port's
     # security groups. If L3 is used, then the pods created will include it.
@@ -453,12 +456,19 @@ function configure_neutron_defaults {
              quota set --secgroups 100 --secgroup-rules 100 "$project_id"
     fi
 
+    # NOTE(dulek): DevStack's admin default for SG's and instances is 10, this
+    #              is too little for our tests with Octavia configured to use
+    #              amphora.
+    openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
+        quota set --secgroups 100 --secgroup-rules 100 --instances 100 admin
+
     if [ -n "$OVS_BRIDGE" ]; then
         iniset "$KURYR_CONFIG" neutron_defaults ovs_bridge "$OVS_BRIDGE"
     fi
     iniset "$KURYR_CONFIG" neutron_defaults external_svc_net "$ext_svc_net_id"
     iniset "$KURYR_CONFIG" octavia_defaults member_mode "$KURYR_K8S_OCTAVIA_MEMBER_MODE"
     iniset "$KURYR_CONFIG" octavia_defaults sg_mode "$KURYR_K8S_OCTAVIA_SG_MODE"
+    iniset "$KURYR_CONFIG" octavia_defaults enforce_sg_rules "$KURYR_ENFORCE_SG_RULES"
     # Octavia takes a very long time to start the LB in the gate. We need
     # to tweak the timeout for the LB creation. Let's be generous and give
     # it up to 20 minutes.
@@ -583,9 +593,18 @@ function wait_for_ok_health {
     echo ""
 }
 
+function get_k8s_log_level {
+  if [[ ${ENABLE_DEBUG_LOG_LEVEL} == "True" ]]; then
+    echo "4"
+  else
+    echo "2"
+  fi
+}
+
 function run_k8s_api {
     local service_cidr
     local cluster_ip_range
+    local command
 
     # Runs Hyperkube's Kubernetes API Server
     wait_for "etcd" "http://${SERVICE_HOST}:${ETCD_PORT}/v2/machines"
@@ -600,7 +619,6 @@ function run_k8s_api {
         cluster_ip_range="$service_cidr"
     fi
 
-    local command
     command=(--net=host
              --volume=${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw)
     if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
@@ -612,7 +630,6 @@ function run_k8s_api {
                 --insecure-bind-address=0.0.0.0
                 --insecure-port=${KURYR_K8S_API_PORT}
                 --etcd-servers=http://${SERVICE_HOST}:${ETCD_PORT}
-                --admission-control=NamespaceLifecycle,LimitRanger,ServiceAccount,ResourceQuota
                 --client-ca-file=/srv/kubernetes/ca.crt
                 --basic-auth-file=/srv/kubernetes/basic_auth.csv
                 --min-request-timeout=300
@@ -620,17 +637,18 @@ function run_k8s_api {
                 --tls-private-key-file=/srv/kubernetes/server.key
                 --token-auth-file=/srv/kubernetes/known_tokens.csv
                 --allow-privileged=true
-                --v=2
+                --v=$(get_k8s_log_level)
                 --logtostderr=true)
 
     run_container kubernetes-api "${command[@]}"
 }
 
 function run_k8s_controller_manager {
+    local command
+
     # Runs Hyperkube's Kubernetes controller manager
     wait_for "Kubernetes API Server" "$KURYR_K8S_API_URL"
 
-    local command
     command=(--net=host
              --volume=${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw)
     if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
@@ -642,17 +660,18 @@ function run_k8s_controller_manager {
                 --service-account-private-key-file=/srv/kubernetes/server.key
                 --root-ca-file=/srv/kubernetes/ca.crt
                 --min-resync-period=3m
-                --v=2
+                --v=$(get_k8s_log_level)
                 --logtostderr=true)
 
     run_container kubernetes-controller-manager "${command[@]}"
 }
 
 function run_k8s_scheduler {
+    local command
+
     # Runs Hyperkube's Kubernetes scheduler
     wait_for "Kubernetes API Server" "$KURYR_K8S_API_URL"
 
-    local command
     command=(--net=host
              --volume=${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw)
     if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
@@ -661,7 +680,7 @@ function run_k8s_scheduler {
     command+=(${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}
               /hyperkube scheduler
                 --master=$KURYR_K8S_API_URL
-                --v=2
+                --v=$(get_k8s_log_level)
                 --logtostderr=true)
 
     run_container kubernetes-scheduler "${command[@]}"
@@ -909,7 +928,7 @@ function create_ingress_l7_router {
     local lb_uuid
 
     lb_name=${KURYR_L7_ROUTER_NAME}
-    max_timeout=600
+    max_timeout=1000
     project_id=$(get_or_create_project \
         "$KURYR_NEUTRON_DEFAULT_PROJECT" default)
 
@@ -1005,6 +1024,7 @@ function update_tempest_conf_file {
     if [[ "$KURYR_K8S_CLOUD_PROVIDER" == "True" ]]; then
         iniset $TEMPEST_CONFIG kuryr_kubernetes cloud_provider True
     fi
+    iniset $TEMPEST_CONFIG kuryr_kubernetes validate_crd True
 }
 
 source $DEST/kuryr-kubernetes/devstack/lib/kuryr_kubernetes
