@@ -17,6 +17,7 @@ from kuryr.lib import config as lib_config
 from oslo_config import cfg
 from oslo_log import log as logging
 
+from kuryr_kubernetes import constants
 from kuryr_kubernetes import version
 
 LOG = logging.getLogger(__name__)
@@ -34,7 +35,7 @@ daemon_opts = [
     cfg.StrOpt('bind_address',
                help=_('Bind address for CNI daemon HTTP server. It is '
                       'recommened to allow only local connections.'),
-               default='127.0.0.1:50036'),
+               default='127.0.0.1:5036'),
     cfg.IntOpt('worker_num',
                help=_('Maximum number of processes that will be spawned to '
                       'process requests from CNI driver.'),
@@ -154,6 +155,18 @@ k8s_opts = [
     cfg.IntOpt('watch_retry_timeout',
                help=_('Time (in seconds) the watcher retries watching for.'),
                default=60),
+    cfg.IntOpt('watch_connection_timeout',
+               help=_('TCP connection timeout (in seconds) for the watcher '
+                      'connections to K8s API.'),
+               default=30),
+    cfg.IntOpt('watch_read_timeout',
+               help=_('TCP read timeout (in seconds) for the watcher '
+                      'connections to K8s API. This affects reaction to time '
+                      'when there are no events being streamed from K8s API. '
+                      'When too low, Kuryr will reconnect more often. When '
+                      'too high, Kuryr will take longer to reconnect when K8s '
+                      'API stream was being silently broken.'),
+               default=60),
     cfg.ListOpt('enabled_handlers',
                 help=_("The comma-separated handlers that should be "
                        "registered for watching in the pipeline."),
@@ -224,6 +237,11 @@ octavia_defaults = [
                choices=[('create', 'replace the VIP SG with a new one'),
                         ('update', 'add rules to the existing VIP SG')],
                default='update'),
+    cfg.BoolOpt('enforce_sg_rules',
+                help=_("Enable the enforcement of SG rules at the LB SG "
+                       "in case the LB does not maintain the source IP "
+                       "of the caller resource"),
+                default=True),
 ]
 
 cache_defaults = [
@@ -249,6 +267,12 @@ nested_vif_driver_opts = [
 DEFAULT_PHYSNET_SUBNET_MAPPINGS = {}
 DEFAULT_DEVICE_MAPPINGS = []
 sriov_opts = [
+    cfg.StrOpt('kubelet_root_dir',
+               help=_("The root directory of the Kubelet daemon"),
+               default='/var/lib/kubelet'),
+    cfg.BoolOpt('enable_pod_resource_service',
+                help=_("Enable PodResources service"),
+                default=False),
     cfg.DictOpt('default_physnet_subnets',
                 help=_("A mapping of default subnets for certain physnets "
                        "in a form of physnet-name:<SUBNET-ID>"),
@@ -260,6 +284,23 @@ sriov_opts = [
                        "physical network names to the agent's node-specific "
                        "physical network device interfaces of SR-IOV physical "
                        "function to be used for VLAN networks.")),
+    cfg.DictOpt('physnet_resource_mappings',
+                help=_("A mapping of physnets for certain sriov dp "
+                       "resource name in a form of "
+                       "physnet-name:resource name. "
+                       "Resource name is listed in sriov device plugin "
+                       "configuation file."),
+                default=DEFAULT_PHYSNET_SUBNET_MAPPINGS),
+    cfg.StrOpt('device_plugin_resource_prefix',
+               help=_("This prefix is used by sriov-network-device-plugin "
+                      "It concatenates with resource suffix defined in "
+                      "sriov device plugin configuration file."),
+               default=constants.K8S_SRIOV_PREFIX),
+    cfg.DictOpt('resource_driver_mappings',
+                help=_("A mappping driver names for certain resource "
+                       "names. Expected that device of VIF related to "
+                       "exact physnet should be binded on specified driver."),
+                default=DEFAULT_PHYSNET_SUBNET_MAPPINGS),
 ]
 
 
