@@ -190,6 +190,17 @@ class PodSubnetsDriver(DriverBase):
         """
         raise NotImplementedError()
 
+    def cleanup_namespace_networks(self, namespace):
+        """Clean up network leftover on the namespace.
+
+        Due to Kuryr controller restarts it may happen that some network
+        resources are leftover. This method ensures they are deleted upon
+        retries.
+
+        :param namespace: name of the Kubernetes namespace object
+        """
+        raise NotImplementedError()
+
 
 @six.add_metaclass(abc.ABCMeta)
 class ServiceSubnetsDriver(DriverBase):
@@ -251,6 +262,8 @@ class PodSecurityGroupsDriver(DriverBase):
         """Create security group rules for a pod.
 
         :param pod: dict containing Kubernetes Pod object
+        :return: a list containing podSelectors of CRDs
+        that had security group rules created
         """
         raise NotImplementedError()
 
@@ -258,6 +271,8 @@ class PodSecurityGroupsDriver(DriverBase):
         """Delete security group rules for a pod
 
         :param pod: dict containing Kubernetes Pod object
+        :return: a list containing podSelectors of CRDs
+        that had security group rules deleted
         """
         raise NotImplementedError()
 
@@ -265,6 +280,8 @@ class PodSecurityGroupsDriver(DriverBase):
         """Update security group rules for a pod
 
         :param pod: dict containing Kubernetes Pod object
+        :return: a list containing podSelectors of CRDs
+        that had security group rules updated
         """
         raise NotImplementedError()
 
@@ -690,7 +707,7 @@ class LBaaSDriver(DriverBase):
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def is_pool_used_by_other_l7policies(l7policy, pool):
+    def is_pool_used_by_other_l7policies(self, l7policy, pool):
         """Checks if pool used by other L7policy.
 
         :param l7policy: `LBaaSL7Policy` object
@@ -699,6 +716,15 @@ class LBaaSDriver(DriverBase):
         """
         raise NotImplementedError()
 
+    @abc.abstractmethod
+    def update_lbaas_sg(self, service, sgs):
+        """Update security group rules associated to the loadbalancer
+
+        :param service: k8s service object
+        :param sgs: list of security group ids to use for updating the rules
+        """
+        raise NotImplementedError()
+
 
 @six.add_metaclass(abc.ABCMeta)
 class VIFPoolDriver(PodVIFDriver):
