@@ -14,12 +14,19 @@
 #    under the License.
 
 import random
+from six.moves import http_client as httplib
 import time
 
 import requests
 
 from neutronclient.common import exceptions as n_exc
 from openstack import exceptions as o_exc
+from openstack.load_balancer.v2 import l7_policy as o_l7p
+from openstack.load_balancer.v2 import l7_rule as o_l7r
+from openstack.load_balancer.v2 import listener as o_lis
+from openstack.load_balancer.v2 import load_balancer as o_lb
+from openstack.load_balancer.v2 import member as o_mem
+from openstack.load_balancer.v2 import pool as o_pool
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import timeutils
@@ -209,12 +216,12 @@ class LBaaSv2Driver(base.LBaaSDriver):
                 for rule in rules['security_group_rules']:
                     # copying ingress rules with same protocol onto the
                     # loadbalancer sg rules
-                    # NOTE(ltomasbo): NP security groups only have
-                    # remote_ip_prefix, not remote_group_id, therefore only
-                    # applying the ones with remote_ip_prefix
+                    # NOTE(ltomasbo): NP sg can only have rules with
+                    # or without remote_ip_prefix. Rules with remote_group_id
+                    # are not possible, therefore only applying the ones
+                    # with or without remote_ip_prefix.
                     if (rule['protocol'] == protocol.lower() and
-                            rule['direction'] == 'ingress' and
-                            rule['remote_ip_prefix']):
+                            rule['direction'] == 'ingress'):
                         # If listener port not in allowed range, skip
                         min_port = rule.get('port_range_min')
                         max_port = rule.get('port_range_max')
@@ -247,8 +254,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
         for rule in lbaas_sg_rules['security_group_rules']:
             if (rule.get('protocol') != protocol.lower() or
                     rule.get('port_range_min') != port or
-                    rule.get('direction') != 'ingress' or
-                    not rule.get('remote_ip_prefix')):
+                    rule.get('direction') != 'ingress'):
                 if all_pod_rules and self._is_default_rule(rule):
                     LOG.debug("Removing default LBaaS sg rule for sg: %r",
                               lb_sg)
@@ -285,7 +291,8 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
     def _is_default_rule(self, rule):
         if (rule.get('direction') == 'ingress' and
-                not rule.get('remote_ip_prefix')):
+                not rule.get('remote_ip_prefix') and
+                'network-policy' not in rule.get('description')):
             return True
         return False
 
@@ -396,10 +403,11 @@ class LBaaSv2Driver(base.LBaaSDriver):
             CONF.kubernetes.service_security_groups_driver == 'namespace')
         create_sg = CONF.octavia_defaults.sg_mode == 'create'
 
-        if namespace_isolation and service_type == 'ClusterIP':
-            self._extend_lb_security_group_rules(loadbalancer, listener)
-        elif create_sg:
+        if create_sg:
             self._create_lb_security_group_rule(loadbalancer, listener)
+        if (namespace_isolation and service_type == 'ClusterIP' and
+                CONF.octavia_defaults.enforce_sg_rules):
+            self._extend_lb_security_group_rules(loadbalancer, listener)
 
     def ensure_listener(self, loadbalancer, protocol, port,
                         service_type='ClusterIP'):
@@ -413,7 +421,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
             result = self._ensure_provisioned(
                 loadbalancer, listener, self._create_listener,
                 self._find_listener, _LB_STS_POLL_SLOW_INTERVAL)
-        except o_exc.BadRequestException:
+        except requests.exceptions.HTTPError:
             LOG.info("Listener creation failed, most probably because "
                      "protocol %(prot)s is not supported", {'prot': protocol})
             return None
@@ -515,7 +523,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
         return None
 
-    def _post_lb_resource(self, path, resource, request):
+    def _post_lb_resource(self, resource, request, **kwargs):
         # FIXME(dulek): openstacksdk doesn't support Octavia tags until version
         #               0.24.0 (Stein+). At the moment our dependency is
         #               >=0.13.0, because we want Kuryr to support multiple
@@ -526,12 +534,14 @@ class LBaaSv2Driver(base.LBaaSDriver):
         #               use lbaas.create_*() directly. Until then we manually
         #               send POST request.
         lbaas = clients.get_loadbalancer_client()
-        response = lbaas.post(path, json={resource: request})
+        response = lbaas.post(resource.base_path % kwargs,
+                              json={resource.resource_key: request})
         if not response.ok:
-            LOG.error('Error when creating %s: %s', resource, response.text)
+            LOG.error('Error when creating %s: %s', resource.resource_key,
+                      response.text)
             response.raise_for_status()
-        response = response.json()[resource]
-        return response
+        response_dict = response.json()[resource.resource_key]
+        return resource(**response_dict)
 
     def _create_loadbalancer(self, loadbalancer):
         request = {
@@ -546,18 +556,16 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
         self._add_tags('loadbalancer', request)
 
-        response = self._post_lb_resource('loadbalancers', 'loadbalancer',
-                                          request)
+        response = self._post_lb_resource(o_lb.LoadBalancer, request)
 
-        loadbalancer.id = response['id']
+        loadbalancer.id = response.id
         loadbalancer.port_id = self._get_vip_port(loadbalancer).get("id")
         if (loadbalancer.provider is not None and
-                loadbalancer.provider != response['provider']):
+                loadbalancer.provider != response.provider):
             LOG.error("Request provider(%s) != Response provider(%s)",
-                      loadbalancer.provider,
-                      response['provider'])
+                      loadbalancer.provider, response.provider)
             return None
-        loadbalancer.provider = response['provider']
+        loadbalancer.provider = response.provider
         return loadbalancer
 
     def _find_loadbalancer(self, loadbalancer):
@@ -570,10 +578,10 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
         try:
             os_lb = next(response)  # openstacksdk returns a generator
-            loadbalancer.id = os_lb['id']
+            loadbalancer.id = os_lb.id
             loadbalancer.port_id = self._get_vip_port(loadbalancer).get("id")
-            loadbalancer.provider = os_lb['provider']
-            if os_lb['provisioning_status'] == 'ERROR':
+            loadbalancer.provider = os_lb.provider
+            if os_lb.provisioning_status == 'ERROR':
                 self.release_loadbalancer(loadbalancer)
                 return None
         except (KeyError, StopIteration):
@@ -590,8 +598,8 @@ class LBaaSv2Driver(base.LBaaSDriver):
             'protocol_port': listener.port,
         }
         self._add_tags('listener', request)
-        response = self._post_lb_resource('listeners', 'listener', request)
-        listener.id = response['id']
+        response = self._post_lb_resource(o_lis.Listener, request)
+        listener.id = response.id
         return listener
 
     def _find_listener(self, listener):
@@ -605,7 +613,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
         try:
             os_listener = next(response)
-            listener.id = os_listener['id']
+            listener.id = os_listener.id
         except (KeyError, StopIteration):
             return None
 
@@ -623,8 +631,8 @@ class LBaaSv2Driver(base.LBaaSDriver):
             'lb_algorithm': lb_algorithm,
         }
         self._add_tags('pool', request)
-        response = self._post_lb_resource('pools', 'pool', request)
-        pool.id = response['id']
+        response = self._post_lb_resource(o_pool.Pool, request)
+        pool.id = response.id
         return pool
 
     def _find_pool(self, pool, by_listener=True):
@@ -638,11 +646,11 @@ class LBaaSv2Driver(base.LBaaSDriver):
         try:
             if by_listener:
                 pools = [p for p in response if pool.listener_id
-                         in {l['id'] for l in p['listeners']}]
+                         in {l['id'] for l in p.listeners}]
             else:
-                pools = [p for p in response if pool.name == p['name']]
+                pools = [p for p in response if pool.name == p.name]
 
-            pool.id = pools[0]['id']
+            pool.id = pools[0].id
         except (KeyError, IndexError):
             return None
         return pool
@@ -659,9 +667,9 @@ class LBaaSv2Driver(base.LBaaSDriver):
             'protocol_port': member.port,
         }
         self._add_tags('member', request)
-        response = self._post_lb_resource('pools/%s/members' % member.pool_id,
-                                          'member', request)
-        member.id = response['id']
+        response = self._post_lb_resource(o_mem.Member, request,
+                                          pool_id=member.pool_id)
+        member.id = response.id
         return member
 
     def _find_member(self, member):
@@ -675,7 +683,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
             protocol_port=member.port)
 
         try:
-            member.id = next(response)['id']
+            member.id = next(response).id
         except (KeyError, StopIteration):
             return None
 
@@ -708,8 +716,9 @@ class LBaaSv2Driver(base.LBaaSDriver):
                 result = self._ensure(obj, create, find)
                 if result:
                     return result
-            except o_exc.BadRequestException:
-                continue
+            except requests.exceptions.HTTPError as e:
+                if e.response.status_code == httplib.BAD_REQUEST:
+                    raise
 
         raise k_exc.ResourceNotReady(obj)
 
@@ -721,7 +730,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
                     return
                 except (o_exc.ConflictException, o_exc.BadRequestException):
                     self._wait_for_provisioning(loadbalancer, remaining)
-            except o_exc.ResourceNotFound:
+            except o_exc.NotFoundException:
                 return
 
         raise k_exc.ResourceNotReady(obj)
@@ -732,7 +741,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
         for remaining in self._provisioning_timer(timeout, interval):
             response = lbaas.get_load_balancer(loadbalancer.id)
-            status = response['provisioning_status']
+            status = response.provisioning_status
             if status == 'ACTIVE':
                 LOG.debug("Provisioning complete for %(lb)s", {
                     'lb': loadbalancer})
@@ -804,14 +813,14 @@ class LBaaSv2Driver(base.LBaaSDriver):
             return None
 
         return obj_lbaas.LBaaSLoadBalancer(
-            id=response['id'],
-            port_id=response['vip_port_id'],
-            name=response['name'],
-            project_id=response['project_id'],
-            subnet_id=response['vip_subnet_id'],
-            ip=response['vip_address'],
+            id=response.id,
+            port_id=response.vip_port_id,
+            name=response.name,
+            project_id=response.project_id,
+            subnet_id=response.vip_subnet_id,
+            ip=response.vip_address,
             security_groups=None,
-            provider=response['provider'])
+            provider=response.provider)
 
     def get_pool_by_name(self, pool_name, project_id):
         lbaas = clients.get_loadbalancer_client()
@@ -824,14 +833,14 @@ class LBaaSv2Driver(base.LBaaSDriver):
         for entry in pools:
             if not entry:
                 continue
-            if entry['name'] == pool_name:
-                listener_id = (entry['listeners'][0]['id'] if
-                               entry['listeners'] else None)
+            if entry.name == pool_name:
+                listener_id = (entry.listeners[0].id if
+                               entry.listeners else None)
                 return obj_lbaas.LBaaSPool(
-                    name=entry['name'], project_id=entry['project_id'],
-                    loadbalancer_id=entry['loadbalancers'][0]['id'],
+                    name=entry.name, project_id=entry.project_id,
+                    loadbalancer_id=entry.loadbalancers[0].id,
                     listener_id=listener_id,
-                    protocol=entry['protocol'], id=entry['id'])
+                    protocol=entry.protocol, id=entry.id)
         return None
 
     def ensure_l7_policy(self, namespace, route_name,
@@ -862,7 +871,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
             'redirect_pool_id': l7_policy.redirect_pool_id,
         }
         self._add_tags('l7policy', request)
-        response = self._post_lb_resource('l7policies', 'l7policy', request)
+        response = self._post_lb_resource(o_l7p.L7Policy, request)
         l7_policy.id = response['id']
         return l7_policy
 
@@ -874,7 +883,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
             redirect_pool_id=l7_policy.redirect_pool_id,
             listener_id=l7_policy.listener_id)
         try:
-            l7_policy.id = next(response)['id']
+            l7_policy.id = next(response).id
         except (KeyError, StopIteration):
             return None
         return l7_policy
@@ -896,8 +905,8 @@ class LBaaSv2Driver(base.LBaaSDriver):
             'value': l7_rule.value
         }
         self._add_tags('rule', request)
-        response = self._post_lb_resource(
-            'l7policies/%s/rules' % l7_rule.l7policy_id, 'rule', request)
+        response = self._post_lb_resource(o_l7r.L7Rule, request,
+                                          l7policy_id=l7_rule.l7policy_id)
         l7_rule.id = response['id']
         return l7_rule
 
@@ -909,7 +918,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
             value=l7_rule.value,
             compare_type=l7_rule.compare_type)
         try:
-            l7_rule.id = next(response)['id']
+            l7_rule.id = next(response).id
         except (KeyError, StopIteration):
             return None
         return l7_rule
@@ -936,8 +945,8 @@ class LBaaSv2Driver(base.LBaaSDriver):
         for entry in l7policy_list:
             if not entry:
                 continue
-            if (entry['redirect_pool_id'] == pool.id and
-                    entry['id'] != l7policy.id):
+            if (entry.redirect_pool_id == pool.id and
+                    entry.id != l7policy.id):
                 return True
         return False
 
