@@ -44,11 +44,14 @@ class NamespacePodSubnetDriver(default_subnet.DefaultPodSubnetDriver):
 
     def get_subnets(self, pod, project_id):
         pod_namespace = pod['metadata']['namespace']
-        subnet_id = self._get_namespace_subnet(pod_namespace)
+        return self.get_namespace_subnet(pod_namespace)
 
+    def get_namespace_subnet(self, namespace, subnet_id=None):
+        if not subnet_id:
+            subnet_id = self._get_namespace_subnet_id(namespace)
         return {subnet_id: utils.get_subnet(subnet_id)}
 
-    def _get_namespace_subnet(self, namespace):
+    def _get_namespace_subnet_id(self, namespace):
         kubernetes = clients.get_kubernetes_client()
         try:
             ns = kubernetes.get('%s/namespaces/%s' % (constants.K8S_API_BASE,
@@ -75,31 +78,47 @@ class NamespacePodSubnetDriver(default_subnet.DefaultPodSubnetDriver):
         return net_crd['spec']['subnetId']
 
     def delete_namespace_subnet(self, net_crd):
-        neutron = clients.get_neutron_client()
-
-        router_id = oslo_cfg.CONF.namespace_subnet.pod_router
         subnet_id = net_crd['spec']['subnetId']
         net_id = net_crd['spec']['netId']
 
-        try:
-            neutron.remove_interface_router(router_id,
-                                            {"subnet_id": subnet_id})
-        except n_exc.NotFound:
-            LOG.debug("Subnet %(subnet)s not attached to router %(router)s",
-                      {'subnet': subnet_id, 'router': router_id})
-        except n_exc.NeutronClientException:
-            LOG.exception("Error deleting subnet %(subnet)s from router "
-                          "%(router)s.", {'subnet': subnet_id, 'router':
-                                          router_id})
-            raise
+        self._delete_namespace_network_resources(subnet_id, net_id)
+
+    def _delete_namespace_network_resources(self, subnet_id, net_id):
+        neutron = clients.get_neutron_client()
+        if subnet_id:
+            router_id = oslo_cfg.CONF.namespace_subnet.pod_router
+            try:
+                neutron.remove_interface_router(router_id,
+                                                {"subnet_id": subnet_id})
+            except n_exc.NotFound:
+                LOG.debug("Subnet %(subnet)s not attached to router "
+                          "%(router)s", {'subnet': subnet_id,
+                                         'router': router_id})
+            except n_exc.NeutronClientException:
+                LOG.exception("Error deleting subnet %(subnet)s from router "
+                              "%(router)s.", {'subnet': subnet_id, 'router':
+                                              router_id})
+                raise
 
         try:
             neutron.delete_network(net_id)
         except n_exc.NotFound:
             LOG.debug("Neutron Network not found: %s", net_id)
         except n_exc.NetworkInUseClient:
-            LOG.exception("One or more ports in use on the network %s.",
-                          net_id)
+            LOG.exception("One or more ports in use on the network %s. "
+                          "Deleting leftovers ports before retrying", net_id)
+            leftover_ports = c_utils.get_ports_by_attrs(status='DOWN',
+                                                        network_id=net_id)
+            for leftover_port in leftover_ports:
+                try:
+                    neutron.delete_port(leftover_port['id'])
+                except n_exc.PortNotFoundClient:
+                    LOG.debug("Port already deleted.")
+                except n_exc.NeutronClientException:
+                    LOG.debug("Unexpected error deleting leftover port %s. "
+                              "Skiping it and continue with the other rest.",
+                              leftover_port['id'])
+                    continue
             raise exceptions.ResourceNotReady(net_id)
         except n_exc.NeutronClientException:
             LOG.exception("Error deleting network %s.", net_id)
@@ -125,17 +144,23 @@ class NamespacePodSubnetDriver(default_subnet.DefaultPodSubnetDriver):
             c_utils.tag_neutron_resources('networks', [neutron_net['id']])
 
             # create a subnet within that network
-            neutron_subnet = neutron.create_subnet(
-                {
-                    "subnet": {
-                        "network_id": neutron_net['id'],
-                        "ip_version": 4,
-                        "name": subnet_name,
-                        "enable_dhcp": False,
-                        "subnetpool_id": subnet_pool_id,
-                        "project_id": project_id
-                    }
-                }).get('subnet')
+            try:
+                neutron_subnet = neutron.create_subnet(
+                    {
+                        "subnet": {
+                            "network_id": neutron_net['id'],
+                            "ip_version": 4,
+                            "name": subnet_name,
+                            "enable_dhcp": False,
+                            "subnetpool_id": subnet_pool_id,
+                            "project_id": project_id
+                        }
+                    }).get('subnet')
+            except n_exc.Conflict:
+                LOG.debug("Max number of retries on neutron side achieved, "
+                          "raising ResourceNotReady to retry subnet creation "
+                          "for %s", subnet_name)
+                raise exceptions.ResourceNotReady(subnet_name)
             c_utils.tag_neutron_resources('subnets', [neutron_subnet['id']])
 
             # connect the subnet to the router
@@ -162,3 +187,22 @@ class NamespacePodSubnetDriver(default_subnet.DefaultPodSubnetDriver):
                           "%(net_id)s, created for the namespace: "
                           "%(namespace)s." % {'net_id': net_crd_spec['netId'],
                                               'namespace': namespace})
+
+    def cleanup_namespace_networks(self, namespace):
+        neutron = clients.get_neutron_client()
+        net_name = 'ns/' + namespace + '-net'
+        filters = {'name': net_name}
+        tags = oslo_cfg.CONF.neutron_defaults.resource_tags
+        if tags:
+            filters['tags'] = tags
+        networks = neutron.list_networks(**filters)['networks']
+        if networks:
+            for net in networks:
+                net_id = net['id']
+                subnets = net.get('subnets')
+                subnet_id = None
+                if subnets:
+                    # NOTE(ltomasbo): Each network created by kuryr only has
+                    # one subnet
+                    subnet_id = subnets[0]
+                self._delete_namespace_network_resources(subnet_id, net_id)
