@@ -218,7 +218,20 @@ def delete_security_group_rule(security_group_rule_id):
         raise
 
 
-def patch_kuryr_crd(crd, i_rules, e_rules, pod_selector, np_spec=None):
+def patch_kuryrnet_crd(crd, populated=True):
+    kubernetes = clients.get_kubernetes_client()
+    crd_name = crd['metadata']['name']
+    LOG.debug('Patching KuryrNet CRD %s' % crd_name)
+    try:
+        kubernetes.patch_crd('spec', crd['metadata']['selfLink'],
+                             {'populated': populated})
+    except k_exc.K8sClientException:
+        LOG.exception('Error updating kuryrnet CRD %s', crd_name)
+        raise
+
+
+def patch_kuryrnetworkpolicy_crd(crd, i_rules, e_rules, pod_selector,
+                                 np_spec=None):
     kubernetes = clients.get_kubernetes_client()
     crd_name = crd['metadata']['name']
     if not np_spec:
@@ -236,9 +249,10 @@ def patch_kuryr_crd(crd, i_rules, e_rules, pod_selector, np_spec=None):
 
 
 def create_security_group_rule_body(
-        security_group_id, direction, port_range_min,
+        security_group_id, direction, port_range_min=None,
         port_range_max=None, protocol=None, ethertype='IPv4', cidr=None,
-        description="Kuryr-Kubernetes NetPolicy SG rule", namespace=None):
+        description="Kuryr-Kubernetes NetPolicy SG rule", namespace=None,
+        pods=None):
     if not port_range_min:
         port_range_min = 1
         port_range_max = 65535
@@ -262,15 +276,18 @@ def create_security_group_rule_body(
             u'remote_ip_prefix'] = cidr
     if namespace:
         security_group_rule_body['namespace'] = namespace
+    if pods:
+        security_group_rule_body['remote_ip_prefixes'] = pods
     LOG.debug("Creating sg rule body %s", security_group_rule_body)
     return security_group_rule_body
 
 
 @MEMOIZE
 def get_pod_ip(pod):
-    vif = pod['metadata']['annotations'].get('openstack.org/kuryr-vif')
-    if vif is None:
-        return vif
+    try:
+        vif = pod['annotations'][constants.K8S_ANNOTATION_VIF]
+    except KeyError:
+        raise k_exc.ResourceNotReady(pod['metadata']['name'])
     vif = jsonutils.loads(vif)
     vif = vif['versioned_object.data']['default_vif']
     network = (vif['versioned_object.data']['network']
@@ -282,14 +299,19 @@ def get_pod_ip(pod):
     return first_subnet_ip
 
 
-def get_annotated_labels(resource, annotation_labels):
+def get_annotations(resource, annotation):
     try:
         annotations = resource['metadata']['annotations']
-        labels_annotation = annotations[annotation_labels]
+        return annotations[annotation]
     except KeyError:
         return None
-    labels = jsonutils.loads(labels_annotation)
-    return labels
+
+
+def get_annotated_labels(resource, annotation_labels):
+    labels_annotation = get_annotations(resource, annotation_labels)
+    if labels_annotation:
+        return jsonutils.loads(labels_annotation)
+    return None
 
 
 def get_kuryrnetpolicy_crds(namespace=None):
@@ -394,15 +416,18 @@ def tag_neutron_resources(resource, res_ids):
                             exc_info=True)
 
 
-def get_services(namespace):
+def get_services(namespace=None):
     kubernetes = clients.get_kubernetes_client()
     try:
-        services = kubernetes.get(
-            '{}/namespaces/{}/services'.format(constants.K8S_API_BASE,
-                                               namespace))
+        if namespace:
+            services = kubernetes.get(
+                '{}/namespaces/{}/services'.format(constants.K8S_API_BASE,
+                                                   namespace))
+        else:
+            services = kubernetes.get(
+                '{}/services'.format(constants.K8S_API_BASE))
     except k_exc.K8sClientException:
-        LOG.exception('Exception when getting K8s services in '
-                      'namespace %s', namespace)
+        LOG.exception('Exception when getting K8s services.')
         raise
     return services
 
@@ -423,3 +448,101 @@ def service_matches_affected_pods(service, pod_selectors):
         if match_selector(selector, svc_selector):
             return True
     return False
+
+
+def get_namespaced_pods(namespace=None):
+    kubernetes = clients.get_kubernetes_client()
+    if namespace:
+        namespace = namespace['metadata']['name']
+        pods = kubernetes.get(
+            '{}/namespaces/{}/pods'.format(
+                constants.K8S_API_BASE, namespace))
+    else:
+        pods = kubernetes.get(
+            '{}/pods'.format(
+                constants.K8S_API_BASE))
+    return pods
+
+
+def get_container_ports(containers, np_port_name, pod):
+    matched_ports = []
+    if is_host_network(pod):
+        return matched_ports
+    for container in containers:
+        for container_port in container.get('ports', []):
+            if container_port.get('name') == np_port_name:
+                container_port = container_port.get('containerPort')
+                if container_port not in matched_ports:
+                    matched_ports.append((pod, container_port))
+    return matched_ports
+
+
+def get_ports(resource, port):
+    """Returns values of ports that have a given port name
+
+    Retrieves the values of ports, defined in the containers
+    associated to the resource, that has its name matching a
+    given port.
+
+    param resource: k8s Pod or Namespace
+    param port: a dict containing a port and protocol
+    return: A list of tuples of port values and associated pods
+    """
+    containers = resource['spec'].get('containers')
+    ports = []
+    np_port = port.get('port')
+    if containers:
+        ports.extend(get_container_ports(containers, np_port, resource))
+    else:
+        pods = get_namespaced_pods(resource).get('items')
+        for pod in pods:
+            containers = pod['spec']['containers']
+            ports.extend(get_container_ports(
+                containers, np_port, pod))
+    return ports
+
+
+def get_namespace(namespace_name):
+    kubernetes = clients.get_kubernetes_client()
+    return kubernetes.get(
+        '{}/namespaces/{}'.format(
+            constants.K8S_API_BASE, namespace_name))
+
+
+def update_port_pci_info(pod, vif):
+    node = get_host_id(pod)
+    annot_port_pci_info = get_port_annot_pci_info(node, vif.id)
+    neutron = clients.get_neutron_client()
+    LOG.debug("Neutron port %s is updated with binding:profile info %s",
+              vif.id, annot_port_pci_info)
+    neutron.update_port(
+        vif.id,
+        {
+            "port": {
+                'binding:profile': annot_port_pci_info
+            }
+        })
+
+
+def get_port_annot_pci_info(nodename, neutron_port):
+    k8s = clients.get_kubernetes_client()
+    annot_name = constants.K8S_ANNOTATION_NODE_PCI_DEVICE_INFO
+    annot_name = annot_name + '-' + neutron_port
+
+    node_info = k8s.get('/api/v1/nodes/{}'.format(nodename))
+    annotations = node_info['metadata']['annotations']
+    try:
+        json_pci_info = annotations[annot_name]
+        pci_info = jsonutils.loads(json_pci_info)
+    except KeyError:
+        pci_info = {}
+    except Exception:
+        LOG.exception('Exception when reading annotations '
+                      '%s and converting from json', annot_name)
+    return pci_info
+
+
+def get_ports_by_attrs(**attrs):
+    neutron = clients.get_neutron_client()
+    ports = neutron.list_ports(**attrs)
+    return ports['ports']
