@@ -16,6 +16,7 @@
 import abc
 import collections
 import eventlet
+import os
 import six
 import time
 
@@ -44,7 +45,7 @@ LOG = logging.getLogger(__name__)
 # Moved out from neutron_default group
 vif_pool_driver_opts = [
     oslo_cfg.IntOpt('ports_pool_max',
-                    help=_("Set a maximun amount of ports per pool. "
+                    help=_("Set a maximum amount of ports per pool. "
                            "0 to disable"),
                     default=0),
     oslo_cfg.IntOpt('ports_pool_min',
@@ -54,7 +55,7 @@ vif_pool_driver_opts = [
                     help=_("Number of ports to be created in a bulk request"),
                     default=10),
     oslo_cfg.IntOpt('ports_pool_update_frequency',
-                    help=_("Minimun interval (in seconds) "
+                    help=_("Minimum interval (in seconds) "
                            "between pool updates"),
                     default=20),
     oslo_cfg.DictOpt('pools_vif_drivers',
@@ -191,8 +192,8 @@ class BaseVIFPool(base.VIFPoolDriver):
         try:
             host_addr = self._get_host_addr(pod)
         except KeyError:
-            LOG.warning("Pod has not been scheduled yet.")
-            raise
+            return None
+
         pool_key = self._get_pool_key(host_addr, project_id, None, subnets)
 
         try:
@@ -261,11 +262,6 @@ class BaseVIFPool(base.VIFPoolDriver):
     def _recover_precreated_ports(self):
         raise NotImplementedError()
 
-    def _get_ports_by_attrs(self, **attrs):
-        neutron = clients.get_neutron_client()
-        ports = neutron.list_ports(**attrs)
-        return ports['ports']
-
     def _get_in_use_ports(self):
         kubernetes = clients.get_kubernetes_client()
         in_use_ports = []
@@ -307,12 +303,20 @@ class BaseVIFPool(base.VIFPoolDriver):
     @lockutils.synchronized('return_to_pool_baremetal')
     @lockutils.synchronized('return_to_pool_nested')
     def sync_pools(self):
+        # NOTE(ltomasbo): Ensure readiness probe is not set to true until the
+        # pools sync is completed in case of controller restart
+        try:
+            os.remove('/tmp/pools_loaded')
+        except OSError:
+            pass
+
         self._available_ports_pools = collections.defaultdict()
         self._existing_vifs = collections.defaultdict()
         self._recyclable_ports = collections.defaultdict()
         self._last_update = collections.defaultdict()
         # NOTE(ltomasbo): Ensure previously created ports are recovered into
         # their respective pools
+        self._cleanup_leftover_ports()
         self._recover_precreated_ports()
 
     def _get_trunks_info(self):
@@ -348,7 +352,11 @@ class BaseVIFPool(base.VIFPoolDriver):
         subports = {}
         subnets = {}
 
-        all_active_ports = self._get_ports_by_attrs(status='ACTIVE')
+        attrs = {'status': 'ACTIVE'}
+        tags = config.CONF.neutron_defaults.resource_tags
+        if tags:
+            attrs['tags'] = tags
+        all_active_ports = c_utils.get_ports_by_attrs(**attrs)
         in_use_ports = self._get_in_use_ports()
 
         for port in all_active_ports:
@@ -375,6 +383,45 @@ class BaseVIFPool(base.VIFPoolDriver):
                                                   subnet_id)}
         return parent_ports, subports, subnets
 
+    def _cleanup_leftover_ports(self):
+        neutron = clients.get_neutron_client()
+        attrs = {'device_owner': kl_const.DEVICE_OWNER, 'status': 'DOWN'}
+        existing_ports = c_utils.get_ports_by_attrs(**attrs)
+
+        tags = config.CONF.neutron_defaults.resource_tags
+        if tags:
+            nets = neutron.list_networks(tags=tags)['networks']
+            nets_ids = [n['id'] for n in nets]
+            for port in existing_ports:
+                net_id = port['network_id']
+                if net_id in nets_ids:
+                    if port.get('binding:host_id'):
+                        for tag in tags:
+                            if tag not in port.get('tags', []):
+                                # delete the port if it has binding details, it
+                                # belongs to the deployment subnet and it does
+                                # not have the right tags
+                                try:
+                                    neutron.delete_port(port['id'])
+                                    break
+                                except n_exc.NeutronClientException:
+                                    LOG.debug("Problem deleting leftover port "
+                                              "%s. Skipping.", port['id'])
+                                    continue
+                    else:
+                        # delete port if they have no binding but belong to the
+                        # deployment networks, regardless of their tagging
+                        try:
+                            neutron.delete_port(port['id'])
+                        except n_exc.NeutronClientException:
+                            LOG.debug("Problem deleting leftover port %s. "
+                                      "Skipping.", port['id'])
+                            continue
+        else:
+            for port in existing_ports:
+                if not port.get('binding:host_id'):
+                    neutron.delete_port(port['id'])
+
 
 class NeutronVIFPool(BaseVIFPool):
     """Manages VIFs for Bare Metal Kubernetes Pods."""
@@ -446,8 +493,8 @@ class NeutronVIFPool(BaseVIFPool):
         name to available_port if the port_debug option is enabled.
         Then the port_id is included in the dict with the available_ports.
 
-        If a maximun number of port per pool is set, the port will be
-        deleted if the maximun has been already reached.
+        If a maximum number of ports per pool is set, the port will be
+        deleted if the maximum has been already reached.
         """
         while True:
             eventlet.sleep(oslo_cfg.CONF.vif_pool.ports_pool_update_frequency)
@@ -462,8 +509,11 @@ class NeutronVIFPool(BaseVIFPool):
         neutron = clients.get_neutron_client()
         sg_current = {}
         if not config.CONF.kubernetes.port_debug:
-            kuryr_ports = self._get_ports_by_attrs(
-                device_owner=kl_const.DEVICE_OWNER)
+            attrs = {'device_owner': kl_const.DEVICE_OWNER}
+            tags = config.CONF.neutron_defaults.resource_tags
+            if tags:
+                attrs['tags'] = tags
+            kuryr_ports = c_utils.get_ports_by_attrs(**attrs)
             for port in kuryr_ports:
                 if port['id'] in self._recyclable_ports:
                     sg_current[port['id']] = tuple(sorted(
@@ -509,13 +559,16 @@ class NeutronVIFPool(BaseVIFPool):
                 LOG.debug('Port already recycled: %s', port_id)
 
     def _recover_precreated_ports(self):
+        attrs = {'device_owner': kl_const.DEVICE_OWNER}
+        tags = config.CONF.neutron_defaults.resource_tags
+        if tags:
+            attrs['tags'] = tags
+
         if config.CONF.kubernetes.port_debug:
-            available_ports = self._get_ports_by_attrs(
-                name=constants.KURYR_PORT_NAME, device_owner=[
-                    kl_const.DEVICE_OWNER])
+            attrs['name'] = constants.KURYR_PORT_NAME
+            available_ports = c_utils.get_ports_by_attrs(**attrs)
         else:
-            kuryr_ports = self._get_ports_by_attrs(
-                device_owner=kl_const.DEVICE_OWNER)
+            kuryr_ports = c_utils.get_ports_by_attrs(**attrs)
             in_use_ports = self._get_in_use_ports()
             available_ports = [port for port in kuryr_ports
                                if port['id'] not in in_use_ports]
@@ -667,8 +720,8 @@ class NestedVIFPool(BaseVIFPool):
         name to available_port if the port_debug option is enabled.
         Then the port_id is included in the dict with the available_ports.
 
-        If a maximun number of ports per pool is set, the port will be
-        deleted if the maximun has been already reached.
+        If a maximum number of ports per pool is set, the port will be
+        deleted if the maximum has been already reached.
         """
         while True:
             eventlet.sleep(oslo_cfg.CONF.vif_pool.ports_pool_update_frequency)
@@ -683,8 +736,11 @@ class NestedVIFPool(BaseVIFPool):
         neutron = clients.get_neutron_client()
         sg_current = {}
         if not config.CONF.kubernetes.port_debug:
-            kuryr_subports = self._get_ports_by_attrs(
-                device_owner=['trunk:subport', kl_const.DEVICE_OWNER])
+            attrs = {'device_owner': ['trunk:subport', kl_const.DEVICE_OWNER]}
+            tags = config.CONF.neutron_defaults.resource_tags
+            if tags:
+                attrs['tags'] = tags
+            kuryr_subports = c_utils.get_ports_by_attrs(**attrs)
             for subport in kuryr_subports:
                 if subport['id'] in self._recyclable_ports:
                     sg_current[subport['id']] = tuple(sorted(
@@ -828,14 +884,32 @@ class NestedVIFPool(BaseVIFPool):
                             LOG.debug('Port %s is not in the available ports '
                                       'pool.', kuryr_subport['id'])
 
+    @lockutils.synchronized('return_to_pool_nested')
+    def populate_pool(self, trunk_ip, project_id, subnets, security_groups):
+        if not hasattr(self, '_available_ports_pools'):
+            LOG.info("Kuryr-controller not yet ready to populate pools.")
+            raise exceptions.ResourceNotReady(trunk_ip)
+        pool_key = self._get_pool_key(trunk_ip, project_id, None, subnets)
+        pools = self._available_ports_pools.get(pool_key)
+        if not pools:
+            # NOTE(ltomasbo): If the amount of nodes is large the repopulation
+            # actions may take too long. Using half of the batch to prevent
+            # the problem
+            num_ports = max(oslo_cfg.CONF.vif_pool.ports_pool_batch/2,
+                            oslo_cfg.CONF.vif_pool.ports_pool_min)
+            self.force_populate_pool(trunk_ip, project_id, subnets,
+                                     security_groups, num_ports)
+
     def force_populate_pool(self, trunk_ip, project_id, subnets,
-                            security_groups, num_ports):
+                            security_groups, num_ports=None):
         """Create a given amount of subports at a given trunk port.
 
         This function creates a given amount of subports and attaches them to
         the specified trunk, adding them to the related subports pool
         regardless of the amount of subports already available in the pool.
         """
+        if not num_ports:
+            num_ports = oslo_cfg.CONF.vif_pool.ports_pool_batch
         vifs = self._drv_vif.request_vifs(
             pod=[],
             project_id=project_id,
@@ -924,8 +998,8 @@ class MultiVIFPool(base.VIFPoolDriver):
         for pod_driver, pool_driver in vif_pool_mapping.items():
             if not utils.check_suitable_multi_pool_driver_opt(pool_driver,
                                                               pod_driver):
-                LOG.error("The pool and pod driver selected are not "
-                          "compatible. They will be skipped")
+                LOG.error("The pool(%s) and pod(%s) driver selected are not "
+                          "compatible.", pool_driver, pod_driver)
                 raise exceptions.MultiPodDriverPoolConfigurationNotSupported()
             drv_vif = base.PodVIFDriver.get_instance(
                 specific_driver=pod_driver)
@@ -998,3 +1072,8 @@ class MultiVIFPool(base.VIFPoolDriver):
                 vif_pool_mapping[pod_driver] = pool_driver
 
         return vif_pool_mapping
+
+    def populate_pool(self, node_ip, project_id, subnets, sg_id):
+        for vif_drv in self._vif_drvs.values():
+            if str(vif_drv) == 'NestedVIFPool':
+                vif_drv.populate_pool(node_ip, project_id, subnets, sg_id)
