@@ -15,7 +15,6 @@
 
 from kuryr.lib._i18n import _
 from oslo_log import log as logging
-from oslo_serialization import jsonutils
 
 from kuryr_kubernetes import clients
 from kuryr_kubernetes import config
@@ -171,7 +170,7 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                 config.CONF.kubernetes.endpoints_driver_octavia_provider)
 
     def on_present(self, endpoints):
-        lbaas_spec = self._get_lbaas_spec(endpoints)
+        lbaas_spec = utils.get_lbaas_spec(endpoints)
         if self._should_ignore(endpoints, lbaas_spec):
             LOG.debug("Ignoring Kubernetes endpoints %s",
                       endpoints['metadata']['name'])
@@ -288,7 +287,15 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
 
         lb = lbaas_state.loadbalancer
         default_sgs = config.CONF.neutron_defaults.pod_security_groups
-        lbaas_spec_sgs = lbaas_spec.security_groups_ids
+        # NOTE(maysams) As the endpoint and svc are annotated with the
+        # 'lbaas_spec' in two separate k8s calls, it's possible that
+        # the endpoint got annotated and the svc haven't due to controller
+        # restarts. For this case, a resourceNotReady exception is raised
+        # till the svc gets annotated with a 'lbaas_spec'.
+        if lbaas_spec:
+            lbaas_spec_sgs = lbaas_spec.security_groups_ids
+        else:
+            raise k_exc.ResourceNotReady(svc_link)
         if lb.security_groups and lb.security_groups != lbaas_spec_sgs:
             sgs = [lb_sg for lb_sg in lb.security_groups
                    if lb_sg not in default_sgs]
@@ -508,14 +515,22 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
 
     def _add_new_listeners(self, endpoints, lbaas_spec, lbaas_state):
         changed = False
-        current_port_tuples = {(listener.protocol, listener.port)
-                               for listener in lbaas_state.listeners}
-        for port_spec in lbaas_spec.ports:
+        lbaas_spec_ports = sorted(lbaas_spec.ports, key=lambda x: x.protocol)
+        for port_spec in lbaas_spec_ports:
             protocol = port_spec.protocol
             port = port_spec.port
-            if (protocol, port) in current_port_tuples:
+            # FIXME (maysams): Due to a bug in Octavia, which does
+            # not allows listeners with same port but different
+            # protocols to co-exist, we need to skip the creation of
+            # listeners that have the same port as an existing one.
+            name = "%s:%s" % (lbaas_state.loadbalancer.name, protocol)
+            listener = self._get_listener_with_same_port(lbaas_state, port)
+            if listener:
+                if listener.protocol != protocol:
+                    LOG.warning("Skipping listener creation for %s "
+                                "as another one already exists with port %r",
+                                name, port)
                 continue
-
             listener = self._drv_lbaas.ensure_listener(
                 loadbalancer=lbaas_state.loadbalancer,
                 protocol=protocol,
@@ -526,6 +541,12 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                 changed = True
         return changed
 
+    def _get_listener_with_same_port(self, lbaas_state, port):
+        for listener in lbaas_state.listeners:
+            if listener.port == port:
+                return listener
+        return None
+
     def _remove_unused_listeners(self, endpoints, lbaas_state, lbaas_spec):
         current_listeners = {p.listener_id for p in lbaas_state.pools}
 
@@ -576,6 +597,9 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
             self._drv_lbaas.release_loadbalancer(
                 loadbalancer=lb)
             lb = None
+            lbaas_state.pools = []
+            lbaas_state.listeners = []
+            lbaas_state.members = []
             changed = True
 
         if not lb:
@@ -600,15 +624,3 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
 
         lbaas_state.loadbalancer = lb
         return changed
-
-    def _get_lbaas_spec(self, endpoints):
-        # TODO(ivc): same as '_get_lbaas_state'
-        try:
-            annotations = endpoints['metadata']['annotations']
-            annotation = annotations[k_const.K8S_ANNOTATION_LBAAS_SPEC]
-        except KeyError:
-            return None
-        obj_dict = jsonutils.loads(annotation)
-        obj = obj_lbaas.LBaaSServiceSpec.obj_from_primitive(obj_dict)
-        LOG.debug("Got LBaaSServiceSpec from annotation: %r", obj)
-        return obj
