@@ -47,7 +47,7 @@ cache.configure_cache_region(oslo_cfg.CONF, namespace_handler_cache_region)
 
 class NamespaceHandler(k8s_base.ResourceEventHandler):
     OBJECT_KIND = constants.K8S_OBJ_NAMESPACE
-    OBJECT_WATCH_PATH = "%s/%s" % (constants.K8S_API_BASE, "namespaces")
+    OBJECT_WATCH_PATH = constants.K8S_API_NAMESPACES
 
     def __init__(self):
         super(NamespaceHandler, self).__init__()
@@ -84,6 +84,12 @@ class NamespaceHandler(k8s_base.ResourceEventHandler):
             self.on_deleted(namespace, net_crd)
             raise exceptions.ResourceNotReady(namespace)
 
+        # NOTE(ltomasbo): Ensure there is no previously created networks
+        # leftovers due to a kuryr-controller crash/restart
+        LOG.debug("Deleting leftovers network resources for namespace: %s",
+                  ns_name)
+        self._drv_subnets.cleanup_namespace_networks(ns_name)
+
         LOG.debug("Creating network resources for namespace: %s", ns_name)
         net_crd_spec = self._drv_subnets.create_namespace_network(ns_name,
                                                                   project_id)
@@ -106,12 +112,12 @@ class NamespaceHandler(k8s_base.ResourceEventHandler):
             net_crd = self._add_kuryrnet_crd(ns_name, net_crd_spec)
             self._set_net_crd(namespace, net_crd)
             self._drv_sg.create_namespace_sg_rules(namespace)
-            self._set_namespace_labels(namespace, current_namespace_labels)
         except exceptions.K8sClientException:
             LOG.exception("Kubernetes client exception. Rolling back "
                           "resources created for the namespace.")
             self._drv_subnets.rollback_network_resources(net_crd_spec, ns_name)
-            self._drv_sg.delete_sg(net_crd_sg['sgId'])
+            if net_crd_sg.get('sgId'):
+                self._drv_sg.delete_sg(net_crd_sg['sgId'])
             self._del_kuryrnet_crd(net_crd_name)
 
     def on_deleted(self, namespace, net_crd=None):
@@ -199,6 +205,11 @@ class NamespaceHandler(k8s_base.ResourceEventHandler):
         kubernetes = clients.get_kubernetes_client()
         net_crd_name = "ns-" + namespace
         spec = {k: v for k, v in net_crd_spec.items()}
+        # NOTE(ltomasbo): To know if the subnet has bee populated with pools.
+        # This is only needed by the kuryrnet handler to skip actions. But its
+        # addition does not have any impact if not used
+        spec['populated'] = False
+
         net_crd = {
             'apiVersion': 'openstack.org/v1',
             'kind': 'KuryrNet',
