@@ -170,8 +170,9 @@ class BaseVIFPool(test_base.TestCase):
         security_groups = [mock.sentinel.security_groups]
         m_driver._get_host_addr.side_effect = KeyError
 
-        self.assertRaises(KeyError, cls.request_vif, m_driver, pod, project_id,
-                          subnets, security_groups)
+        resp = cls.request_vif(m_driver, pod, project_id, subnets,
+                               security_groups)
+        self.assertIsNone(resp)
 
     @mock.patch('time.time', return_value=50)
     @ddt.data((neutron_vif.NeutronPodVIFDriver),
@@ -326,6 +327,124 @@ class BaseVIFPool(test_base.TestCase):
 
         self.assertEqual(resp, [])
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test_cleanup_leftover_ports(self, m_get_ports):
+        cls = vif_pool.BaseVIFPool
+        m_driver = mock.MagicMock(spec=cls)
+        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+
+        port_id = str(uuid.uuid4())
+        port = get_port_obj(port_id=port_id)
+        net_id = port['network_id']
+        tags = 'clusterTest'
+        port['tags'] = [tags]
+        m_get_ports.return_value = [port]
+        oslo_cfg.CONF.set_override('resource_tags',
+                                   tags,
+                                   group='neutron_defaults')
+        neutron.list_networks.return_value = {'networks': [{'id': net_id}]}
+
+        cls._cleanup_leftover_ports(m_driver)
+        neutron.list_networks.assert_called()
+        neutron.delete_port.assert_not_called()
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test_cleanup_leftover_ports_different_network(self, m_get_ports):
+        cls = vif_pool.BaseVIFPool
+        m_driver = mock.MagicMock(spec=cls)
+        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+
+        port_id = str(uuid.uuid4())
+        port = get_port_obj(port_id=port_id)
+        tags = 'clusterTest'
+        port['tags'] = [tags]
+        m_get_ports.return_value = [port]
+        oslo_cfg.CONF.set_override('resource_tags',
+                                   tags,
+                                   group='neutron_defaults')
+        neutron.list_networks.return_value = {'networks': []}
+
+        cls._cleanup_leftover_ports(m_driver)
+        neutron.list_networks.assert_called()
+        neutron.delete_port.assert_not_called()
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test_cleanup_leftover_ports_no_binding(self, m_get_ports):
+        cls = vif_pool.BaseVIFPool
+        m_driver = mock.MagicMock(spec=cls)
+        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+
+        port_id = str(uuid.uuid4())
+        port = get_port_obj(port_id=port_id)
+        net_id = port['network_id']
+        tags = 'clusterTest'
+        port['tags'] = [tags]
+        port['binding:host_id'] = None
+        m_get_ports.return_value = [port]
+        oslo_cfg.CONF.set_override('resource_tags',
+                                   tags,
+                                   group='neutron_defaults')
+        neutron.list_networks.return_value = {'networks': [{'id': net_id}]}
+
+        cls._cleanup_leftover_ports(m_driver)
+        neutron.list_networks.assert_called()
+        neutron.delete_port.assert_called_once_with(port['id'])
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test_cleanup_leftover_ports_no_tags(self, m_get_ports):
+        cls = vif_pool.BaseVIFPool
+        m_driver = mock.MagicMock(spec=cls)
+        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+
+        port_id = str(uuid.uuid4())
+        port = get_port_obj(port_id=port_id)
+        net_id = port['network_id']
+        tags = 'clusterTest'
+        m_get_ports.return_value = [port]
+        oslo_cfg.CONF.set_override('resource_tags',
+                                   tags,
+                                   group='neutron_defaults')
+        neutron.list_networks.return_value = {'networks': [{'id': net_id}]}
+
+        cls._cleanup_leftover_ports(m_driver)
+        neutron.list_networks.assert_called()
+        neutron.delete_port.assert_called_once_with(port['id'])
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test_cleanup_leftover_ports_no_tagging(self, m_get_ports):
+        cls = vif_pool.BaseVIFPool
+        m_driver = mock.MagicMock(spec=cls)
+        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+
+        port_id = str(uuid.uuid4())
+        port = get_port_obj(port_id=port_id)
+        m_get_ports.return_value = [port]
+        oslo_cfg.CONF.set_override('resource_tags',
+                                   [],
+                                   group='neutron_defaults')
+
+        cls._cleanup_leftover_ports(m_driver)
+        neutron.list_networks.assert_not_called()
+        neutron.delete_port.assert_not_called()
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test_cleanup_leftover_ports_no_tagging_no_binding(self, m_get_ports):
+        cls = vif_pool.BaseVIFPool
+        m_driver = mock.MagicMock(spec=cls)
+        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+
+        port_id = str(uuid.uuid4())
+        port = get_port_obj(port_id=port_id)
+        port['binding:host_id'] = None
+        m_get_ports.return_value = [port]
+        oslo_cfg.CONF.set_override('resource_tags',
+                                   [],
+                                   group='neutron_defaults')
+
+        cls._cleanup_leftover_ports(m_driver)
+        neutron.list_networks.assert_not_called()
+        neutron.delete_port.assert_called_once_with(port['id'])
+
 
 @ddt.ddt
 class NeutronVIFPool(test_base.TestCase):
@@ -547,8 +666,9 @@ class NeutronVIFPool(test_base.TestCase):
 
         neutron.update_port.assert_not_called()
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
     @ddt.data((0), (10))
-    def test__trigger_return_to_pool(self, max_pool):
+    def test__trigger_return_to_pool(self, max_pool, m_get_ports):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -565,7 +685,7 @@ class NeutronVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('port_debug',
                                    True,
                                    group='kubernetes')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
 
@@ -581,8 +701,9 @@ class NeutronVIFPool(test_base.TestCase):
             })
         neutron.delete_port.assert_not_called()
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
     @ddt.data((0), (10))
-    def test__trigger_return_to_pool_no_update(self, max_pool):
+    def test__trigger_return_to_pool_no_update(self, max_pool, m_get_ports):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -599,7 +720,7 @@ class NeutronVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('port_debug',
                                    False,
                                    group='kubernetes')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group']}]
         m_driver._get_pool_size.return_value = pool_length
 
@@ -608,7 +729,8 @@ class NeutronVIFPool(test_base.TestCase):
         neutron.update_port.assert_not_called()
         neutron.delete_port.assert_not_called()
 
-    def test__trigger_return_to_pool_delete_port(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test__trigger_return_to_pool_delete_port(self, m_get_ports):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -624,7 +746,7 @@ class NeutronVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('ports_pool_max',
                                    10,
                                    group='vif_pool')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
 
@@ -633,7 +755,8 @@ class NeutronVIFPool(test_base.TestCase):
         neutron.update_port.assert_not_called()
         neutron.delete_port.assert_called_once_with(port_id)
 
-    def test__trigger_return_to_pool_update_exception(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test__trigger_return_to_pool_update_exception(self, m_get_ports):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -653,7 +776,7 @@ class NeutronVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('port_debug',
                                    True,
                                    group='kubernetes')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
         neutron.update_port.side_effect = n_exc.NeutronClientException
@@ -670,7 +793,8 @@ class NeutronVIFPool(test_base.TestCase):
             })
         neutron.delete_port.assert_not_called()
 
-    def test__trigger_return_to_pool_delete_exception(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test__trigger_return_to_pool_delete_exception(self, m_get_ports):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -686,7 +810,7 @@ class NeutronVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('ports_pool_max',
                                    5,
                                    group='vif_pool')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
         neutron.delete_port.side_effect = n_exc.PortNotFoundClient
@@ -696,7 +820,8 @@ class NeutronVIFPool(test_base.TestCase):
         neutron.update_port.assert_not_called()
         neutron.delete_port.assert_called_once_with(port_id)
 
-    def test__trigger_return_to_pool_delete_key_error(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test__trigger_return_to_pool_delete_key_error(self, m_get_ports):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -711,7 +836,7 @@ class NeutronVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('ports_pool_max',
                                    5,
                                    group='vif_pool')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
 
@@ -720,9 +845,11 @@ class NeutronVIFPool(test_base.TestCase):
         neutron.update_port.assert_not_called()
         neutron.delete_port.assert_not_called()
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
     @mock.patch('kuryr_kubernetes.os_vif_util.neutron_to_osvif_vif')
     @mock.patch('kuryr_kubernetes.utils.get_subnet')
-    def test__recover_precreated_ports(self, m_get_subnet, m_to_osvif):
+    def test__recover_precreated_ports(self, m_get_subnet, m_to_osvif,
+                                       m_get_ports):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
 
@@ -736,7 +863,7 @@ class NeutronVIFPool(test_base.TestCase):
         port_id = str(uuid.uuid4())
         port = get_port_obj(port_id=port_id)
         filtered_ports = [port]
-        m_driver._get_ports_by_attrs.return_value = filtered_ports
+        m_get_ports.return_value = filtered_ports
         vif_plugin = mock.sentinel.plugin
         m_driver._drv_vif._get_vif_plugin.return_value = vif_plugin
 
@@ -758,7 +885,7 @@ class NeutronVIFPool(test_base.TestCase):
 
         cls._recover_precreated_ports(m_driver)
 
-        m_driver._get_ports_by_attrs.assert_called_once()
+        m_get_ports.assert_called_once()
         m_get_subnet.assert_called_with(subnet_id)
         m_driver._drv_vif._get_vif_plugin.assert_called_once_with(port)
         m_to_osvif.assert_called_once_with(vif_plugin, port, subnet)
@@ -767,14 +894,16 @@ class NeutronVIFPool(test_base.TestCase):
         self.assertEqual(m_driver._available_ports_pools[pool_key],
                          {tuple(port['security_groups']): [port_id]})
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
     @mock.patch('kuryr_kubernetes.os_vif_util.neutron_to_osvif_vif')
     @mock.patch('kuryr_kubernetes.utils.get_subnet')
-    def test__recover_precreated_ports_empty(self, m_get_subnet, m_to_osvif):
+    def test__recover_precreated_ports_empty(self, m_get_subnet, m_to_osvif,
+                                             m_get_ports):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
 
         filtered_ports = []
-        m_driver._get_ports_by_attrs.return_value = filtered_ports
+        m_get_ports.return_value = filtered_ports
         m_driver._get_trunks_info.return_value = ({}, {}, {})
 
         oslo_cfg.CONF.set_override('port_debug',
@@ -783,7 +912,7 @@ class NeutronVIFPool(test_base.TestCase):
 
         cls._recover_precreated_ports(m_driver)
 
-        m_driver._get_ports_by_attrs.assert_called_once()
+        m_get_ports.assert_called_once()
         m_get_subnet.assert_not_called()
         m_to_osvif.assert_not_called()
 
@@ -1081,8 +1210,9 @@ class NestedVIFPool(test_base.TestCase):
 
         neutron.update_port.assert_not_called()
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
     @ddt.data((0), (10))
-    def test__trigger_return_to_pool(self, max_pool):
+    def test__trigger_return_to_pool(self, max_pool, m_get_ports):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -1099,7 +1229,7 @@ class NestedVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('port_debug',
                                    True,
                                    group='kubernetes')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
 
@@ -1114,8 +1244,9 @@ class NestedVIFPool(test_base.TestCase):
             })
         neutron.delete_port.assert_not_called()
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
     @ddt.data((0), (10))
-    def test__trigger_return_to_pool_no_update(self, max_pool):
+    def test__trigger_return_to_pool_no_update(self, max_pool, m_get_ports):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -1132,7 +1263,7 @@ class NestedVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('port_debug',
                                    False,
                                    group='kubernetes')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group']}]
         m_driver._get_pool_size.return_value = pool_length
 
@@ -1141,7 +1272,8 @@ class NestedVIFPool(test_base.TestCase):
         neutron.update_port.assert_not_called()
         neutron.delete_port.assert_not_called()
 
-    def test__trigger_return_to_pool_delete_port(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test__trigger_return_to_pool_delete_port(self, m_get_ports):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -1162,7 +1294,7 @@ class NestedVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('ports_pool_max',
                                    10,
                                    group='vif_pool')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
         m_driver._get_trunk_id.return_value = trunk_id
@@ -1177,7 +1309,8 @@ class NestedVIFPool(test_base.TestCase):
                                                                   trunk_id,
                                                                   port_id)
 
-    def test__trigger_return_to_pool_update_exception(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test__trigger_return_to_pool_update_exception(self, m_get_ports):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -1194,7 +1327,7 @@ class NestedVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('port_debug',
                                    True,
                                    group='kubernetes')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
         neutron.update_port.side_effect = n_exc.NeutronClientException
@@ -1210,7 +1343,8 @@ class NestedVIFPool(test_base.TestCase):
             })
         neutron.delete_port.assert_not_called()
 
-    def test__trigger_return_to_pool_delete_exception(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test__trigger_return_to_pool_delete_exception(self, m_get_ports):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -1231,7 +1365,7 @@ class NestedVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('ports_pool_max',
                                    5,
                                    group='vif_pool')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
         m_driver._get_trunk_id.return_value = trunk_id
@@ -1247,7 +1381,8 @@ class NestedVIFPool(test_base.TestCase):
                                                                   port_id)
         neutron.delete_port.assert_called_once_with(port_id)
 
-    def test__trigger_return_to_pool_delete_key_error(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test__trigger_return_to_pool_delete_key_error(self, m_get_ports):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -1266,7 +1401,7 @@ class NestedVIFPool(test_base.TestCase):
         oslo_cfg.CONF.set_override('ports_pool_max',
                                    5,
                                    group='vif_pool')
-        m_driver._get_ports_by_attrs.return_value = [
+        m_get_ports.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
         m_driver._known_trunk_ids = {}
@@ -1295,8 +1430,9 @@ class NestedVIFPool(test_base.TestCase):
         self.assertEqual(ip_address, cls._get_parent_port_ip(m_driver,
                                                              port_id))
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
     @mock.patch('kuryr_kubernetes.utils.get_subnet')
-    def test__get_trunk_info(self, m_get_subnet):
+    def test__get_trunk_info(self, m_get_subnet, m_get_ports):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
 
@@ -1314,7 +1450,7 @@ class NestedVIFPool(test_base.TestCase):
         subport_id = str(uuid.uuid4())
         subport = get_port_obj(port_id=subport_id,
                                device_owner='trunk:subport')
-        m_driver._get_ports_by_attrs.return_value = [trunk_port, subport]
+        m_get_ports.return_value = [trunk_port, subport]
         m_driver._get_in_use_ports.return_value = []
         subnet = mock.sentinel.subnet
         m_get_subnet.return_value = subnet
@@ -1331,11 +1467,12 @@ class NestedVIFPool(test_base.TestCase):
         self.assertEqual(r_subports, {subport_id: subport})
         self.assertEqual(r_subnets, exp_subnets)
 
-    def test__get_trunk_info_empty(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test__get_trunk_info_empty(self, m_get_ports):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
 
-        m_driver._get_ports_by_attrs.return_value = []
+        m_get_ports.return_value = []
         m_driver._get_in_use_ports.return_value = []
 
         r_p_ports, r_subports, r_subnets = cls._get_trunks_info(m_driver)
@@ -1344,14 +1481,15 @@ class NestedVIFPool(test_base.TestCase):
         self.assertEqual(r_subports, {})
         self.assertEqual(r_subnets, {})
 
-    def test__get_trunk_info_no_trunk_details(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_ports_by_attrs')
+    def test__get_trunk_info_no_trunk_details(self, m_get_ports):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
 
         port_id = str(uuid.uuid4())
         port = get_port_obj(port_id=port_id)
         port = get_port_obj(port_id=port_id, device_owner='compute:nova')
-        m_driver._get_ports_by_attrs.return_value = [port]
+        m_get_ports.return_value = [port]
         m_driver._get_in_use_ports.return_value = []
 
         r_p_ports, r_subports, r_subnets = cls._get_trunks_info(m_driver)
