@@ -48,15 +48,25 @@ subnet_caching_opts = [
     cfg.IntOpt('cache_time', default=3600),
 ]
 
+nodes_caching_opts = [
+    cfg.BoolOpt('caching', default=True),
+    cfg.IntOpt('cache_time', default=3600),
+]
+
 CONF.register_opts(subnet_caching_opts, "subnet_caching")
+CONF.register_opts(nodes_caching_opts, "nodes_caching")
 
 cache.configure(CONF)
 subnet_cache_region = cache.create_region()
 MEMOIZE = cache.get_memoization_decorator(
     CONF, subnet_cache_region, "subnet_caching")
-
 cache.configure_cache_region(CONF, subnet_cache_region)
 
+nodes_cache_region = cache.create_region()
+MEMOIZE_NODE = cache.get_memoization_decorator(
+    CONF, nodes_cache_region, "nodes_caching")
+cache.configure_cache_region(CONF, nodes_cache_region)
+
 
 def utf8_json_decoder(byte_data):
     """Deserializes the bytes into UTF-8 encoded JSON.
@@ -118,16 +128,16 @@ def exponential_sleep(deadline, attempt, interval=DEFAULT_INTERVAL):
     if seconds_left <= 0:
         return 0
 
-    interval = random.randint(1, 2 ** attempt - 1) * DEFAULT_INTERVAL
+    to_sleep = random.randint(1, 2 ** attempt - 1) * interval
 
-    if interval > seconds_left:
-        interval = seconds_left
+    if to_sleep > seconds_left:
+        to_sleep = seconds_left
 
-    if interval < DEFAULT_INTERVAL:
-        interval = DEFAULT_INTERVAL
+    if to_sleep < interval:
+        to_sleep = interval
 
-    time.sleep(interval)
-    return interval
+    time.sleep(to_sleep)
+    return to_sleep
 
 
 def get_node_name():
@@ -148,6 +158,25 @@ def get_leader_name():
         return None
 
 
+@MEMOIZE_NODE
+def get_nodes_ips():
+    """Get the IPs of the trunk ports associated to the deployment."""
+    trunk_ips = []
+    neutron = clients.get_neutron_client()
+    tags = CONF.neutron_defaults.resource_tags
+    if tags:
+        ports = neutron.list_ports(status='ACTIVE',
+                                   tags=CONF.neutron_defaults.resource_tags)
+    else:
+        # NOTE(ltomasbo: if tags are not used, assume all the trunk ports are
+        # part of the kuryr deployment
+        ports = neutron.list_ports(status='ACTIVE')
+    for port in ports.get('ports'):
+        if port.get('trunk_details'):
+            trunk_ips.append(port['fixed_ips'][0]['ip_address'])
+    return trunk_ips
+
+
 @MEMOIZE
 def get_subnet(subnet_id):
     neutron = clients.get_neutron_client()
@@ -213,9 +242,10 @@ def has_kuryr_crd(crd_url):
     return True
 
 
-def get_lbaas_spec(service):
+def get_lbaas_spec(k8s_object):
+    # k8s_object can be service or endpoint
     try:
-        annotations = service['metadata']['annotations']
+        annotations = k8s_object['metadata']['annotations']
         annotation = annotations[constants.K8S_ANNOTATION_LBAAS_SPEC]
     except KeyError:
         return None
@@ -242,14 +272,22 @@ def set_lbaas_spec(service, lbaas_spec):
     try:
         k8s.annotate(ep_link,
                      {constants.K8S_ANNOTATION_LBAAS_SPEC: annotation})
-    except exceptions.K8sResourceNotFound:
+    except exceptions.K8sResourceNotFound as ex:
+        LOG.debug("Failed to annotate svc: %s", ex)
         raise exceptions.ResourceNotReady(ep_link)
     except exceptions.K8sClientException:
         LOG.debug("Failed to annotate endpoint %r", ep_link)
         raise
-    k8s.annotate(svc_link,
-                 {constants.K8S_ANNOTATION_LBAAS_SPEC: annotation},
-                 resource_version=service['metadata']['resourceVersion'])
+    try:
+        k8s.annotate(svc_link,
+                     {constants.K8S_ANNOTATION_LBAAS_SPEC: annotation},
+                     resource_version=service['metadata']['resourceVersion'])
+    except exceptions.K8sResourceNotFound as ex:
+        LOG.debug("Failed to annotate svc: %s", ex)
+        raise exceptions.ResourceNotReady(svc_link)
+    except exceptions.K8sClientException:
+        LOG.exception("Failed to annotate svc: %r", svc_link)
+        raise
 
 
 def get_lbaas_state(endpoint):
